/**
 * @copyright
 * Copyright 2013 Mail Bypass, Inc.
 *
 * @license
 * Licensed under the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain
 * a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

/**
 * @ignore
 */
var util = require('util'),
    _ = require('underscore'),
    request = require('request'),
    constants = require('./constants');

/**
 * This module contains classes for interfacing with the Message Bus API.
 * @module MessageBus
 */

/**
 * @class MessageBusBase
 * @constructor
 * @protected
 * @param apiKey {String} Api key
 * @param [uri = constants.uri] {String} Host uri
 * @param [timeout = constants.timeout] {Number} Connection timeout in milliseconds
 */
var MessageBusBase = function(apiKey, uri, timeout) {

    /**
     * @property apiKey
     * @type String
     */
    this.apiKey = apiKey;

    /**
     * @property [uri = constants.uri]
     * @type String
     */
    this.uri = uri || constants.uri;

    /**
     * @property [uri = constants.timeout]
     * @type Number
     */
    this.timeout = timeout || constants.timeout;

    /**
     * @property request
     * @private
     * @type request
     */
    this.request = request.defaults({
        timeout: this.timeout,
        jar: false
    });

    /**
     * @property headers
     * @private
     * @type Object
     */
    this.headers = {
        'X-MessageBus-Key': this.apiKey,
        'User-Agent': constants.userAgent
    };

    /**
     * @property [maxSockets = constants.maxSockets]
     * @private
     * @type Number
     */
    this.maxSockets = constants.maxSockets;
};

/**
 * Initiates the REST request to the API server and handles the response(s).
 * @protected
 * @param {String} path - Uri path
 * @param [data = null] - {Object} Request data
 * @param [method = GET] - {String} HTTP request method - GET, PUT, POST, DELETE
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 */
MessageBusBase.prototype.callApi = function(path, data, method, cb) {
    method = method || 'GET';

    if (!_.isFunction(cb)) {
        cb = function() {};
    }

    if (_.isFunction(data)) {
        cb = data;
        data = {};
    }

    if (_.isFunction(method)) {
        cb = method;
        method = 'GET';
    }

    var options = {
        uri: this.uri + path,
        method: method
    };

    if (method === 'GET') {
        options['qs'] = data;
    } else if (method === 'POST' || method === 'PUT') {
        options['body'] = JSON.stringify(data);
        this.headers['Content-Type'] = 'application/json; charset=utf-8';
    }
    options['headers'] = this.headers;

    options['pool'] = {
        maxSockets: this.maxSockets
    };

    this.request(options, function(err, resp, body) {
        if (err) {
            cb(err);
        } else if (!body) {
            cb(new Error('empty response'));
        } else {
            try {
                data = JSON.parse(body);
            } catch (error) {
                cb(new Error('MessageBus failed JSON.parse:' + body));
            }
            if (_.isNull(data) || _.isUndefined(data) || !_.has(data,
                'statusCode')) {
                cb(new Error('undefined response: ' + util.inspect(resp)));
            } else {
                if (data.statusCode > 299) {
                    cb(data);
                } else {
                    cb(null, data);
                }
            }
        }
    });
};


/**
 * Formats and validates the start and end dates in iso8601 format
 * @protected
 * @param [startDate = null] - {Date|String|Number} Start date to use within the query
 * @param [endDate = null] - {Date|String|Number} End date to use within the query
 * @return {Object} - Object with formatted dates
 */
MessageBusBase.prototype.formatDateParams = function(startDate, endDate) {
    var toISO = function(millisecondsSinceEpoch) {
        function pad(number) {
            if (number >= 0 && number <= 9) {
                return "0" + number;
            }
            return "" + number;
        }

        var date = new Date(millisecondsSinceEpoch);
        return date.getUTCFullYear()
            + '-' + pad(date.getUTCMonth() + 1)
            + '-' + pad(date.getUTCDate())
            + 'T' + pad(date.getUTCHours())
            + ':' + pad(date.getUTCMinutes())
            + ':' + pad(date.getUTCSeconds())
            + 'Z';
    };

    var formatDate = function(date) {
        if (_.isDate(date)) return toISO(date);
        else if (!_.isNaN(Date.parse(date))) {
            try {
                return toISO(new Date(date));
            } catch (error) {
                return null;
            }
        } else if (typeof date == 'number') {
            return toISO(date);
        }
    };

    startDate = formatDate(startDate);
    endDate = formatDate(endDate);

    var dateParams = {};
    if (startDate) dateParams['startDate'] = startDate;
    if (endDate) dateParams['endDate'] = endDate;
    return dateParams;
};

/**
 * MessageBusAPIClient sends email and provides methods for channel and session based segmentation.
 * @class MessageBusAPIClient
 * @constructor
 * @param apiKey {String} Api key
 * @param [uri = constants.uri] {String} Host uri
 * @param [timeout = constants.timeout] {Number} Connection timeout in milliseconds
 * @example
 var client = new MessageBusAPIClient('apiKey');
 */
var MessageBusAPIClient = function(apiKey, uri, timeout) {
    MessageBusAPIClient.super_.call(this, apiKey, uri, timeout);
};
util.inherits(MessageBusAPIClient, MessageBusBase);


/**
 * Send emails via the Message Bus API.
 * {@link https://www.messagebus.com/documentation-content/#Sending Email|Send Email REST api}
 * @param messages - {Array} Messages to send
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 var client = new MessageBusAPIClient('apiKey');
 var messages = [
 {
    toEmail: 'bob@example.com',
    fromEmail: 'alice@example.com',
    subject: 'Email #1',
    plaintextBody: 'Hey Bob!'
 },
 {
    toEmail: 'john@example.com',
    fromEmail: 'stefan@example.com',
    subject: 'Email #2',
    plaintextBody: 'Hey John!'
 }];

 client.sendMessages(messages, function (err, resp) {
    if (err) throw err;
    console.info('Emails sent:', resp['successCount']);
    console.info('Emails failed:', resp['failureCount']);
});
 */
MessageBusAPIClient.prototype.sendMessages = function(messages, cb) {
    if (_.isArray(messages) && messages.length <= constants.maxMessages) {
        this.callApi(constants.endPoints.messageEmailSend, {
            messages: messages
        }, 'POST', cb);
    } else {
        var err = new Error(util.format(
            'messages must be an array with maximum of %s messages', constants.maxMessages));
        if (cb) cb(err);
        else throw err;
    }
};

/**
 * Fetches the list of channels associated with the account.
 * {@link https://www.messagebus.com/documentation-content/#Separating Mail Streams/Method 1|Get Channels REST api}
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 var client = new MessageBusAPIClient('apiKey');
 client.getChannels(function (err, resp) {
    if (err) throw err;
    console.info(resp.results);
});
 */
MessageBusAPIClient.prototype.getChannels = function(cb) {
    this.callApi(constants.endPoints.channels, cb);
};

/**
 * Fetches the channel config for a given channel.
 * {@link https://www.messagebus.com/documentation-content/#Separating Mail Streams/Method 2|Get Channel config REST api}
 * @param  channel - {String} Channel key
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 var client = new MessageBusAPIClient('apiKey');
 client.getChannelConfig('channelKey', function (err, resp) {
    if (err) throw err;
    console.info(resp.configuration);
});
 */

MessageBusAPIClient.prototype.getChannelConfig = function(channel, cb) {
    this.callApi(util.format(constants.endPoints.channelConfig, channel), cb);
};

/**
 * Fetches the sessions associated to a channel.
 * {@link https://www.messagebus.com/documentation-content/#Separating Mail Streams/Method 4|Get Sessions REST api}
 * @param  channel - {String}  Channel key
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 var client = new MessageBusAPIClient('apiKey');
 client.getChannelSessions('channelKey', function (err, resp) {
    if (err) throw err;
    console.info(resp.results);
});
 */
MessageBusAPIClient.prototype.getChannelSessions = function(channel, cb) {
    this.callApi(util.format(constants.endPoints.channelSessions, channel), cb);
};

/**
 * Creates a new session within a channel.
 * {@link https://www.messagebus.com/documentation-content/#Separating Mail Streams/Method 3|Create Session REST api}
 * @param  channel - {String} Channel key
 * @param  sessionName  -{String} Session key
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 var client = new MessageBusAPIClient('apiKey');
 client.createSession('channelKey', 'newSessionName', function (err, resp) {
    if (err) throw err;
    console.info(resp.sessionKey);
});
 */
MessageBusAPIClient.prototype.createSession = function(channel, sessionName, cb) {
    this.callApi(util.format(constants.endPoints.channelSessions, channel), {
        sessionName: sessionName
    }, 'POST', cb);
};

/**
 * Renames an existing session.
 * {@link https://www.messagebus.com/documentation-content/#Separating Mail Streams/Method 5|Rename Session REST api}
 * @param channel - {String} Channel key
 * @param session  - {String} Session key
 * @param sessionName - {String} New session name
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 var client = new MessageBusAPIClient('apiKey');
 client.sessionRename('channelKey', 'sessionKey', 'newSessionName', function (err, resp) {
    if (err) throw err;
    console.info(resp);
});
 */
MessageBusAPIClient.prototype.sessionRename = function(channel, session,
    sessionName, cb) {
    this.callApi(util.format(constants.endPoints.channelSessionRename, channel, session), {
        sessionName: sessionName
    }, 'PUT', cb);
};

/**
 * Get the version of the API.
 * {@link https://www.messagebus.com/documentation-content/#Method: version|Version REST api}
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 */
MessageBusAPIClient.prototype.getVersion = function(cb) {
    this.callApi(constants.endPoints.version, cb);
};


/**
 * MessageBusStatsClient provides methods to query stats associated with sending of email.
 * @class MessageBusStatsClient
 * @constructor
 * @param apiKey {String} Api key
 * @param [uri = constants.uri] {String} Host uri
 * @param [timeout = constants.timeout]{Number} Connection timeout in milliseconds
 */
var MessageBusStatsClient = function(apiKey, uri, timeout) {
    MessageBusStatsClient.super_.call(this, apiKey, uri, timeout);
};
util.inherits(MessageBusStatsClient, MessageBusBase);


/**
 * Gets stats associated with an account, channel, session on a time range.
 * {@link https://www.messagebus.com/documentation-content/#Email Metrics|Email Stats REST api}
 * @param [params = null] - {Object} query params
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 var client = new MessageBusStatsClient('apiKey');
 //params and it's keys are optional.
 var params = {startDate: '2012-12-10', endDate: '2013-01-01', channel:'channelKey', session:'sessionKey'};
 client.getStats(params, function (err, resp) {
    if (err) throw err;
    console.info(resp);
});
 */
MessageBusStatsClient.prototype.getStats = function(params, cb) {
    params = params || {};
    if (_.isFunction(params)) {
        cb = params;
        params = {};
    }
    params = _.extend(_.omit(params, 'startDate', 'endDate'),
        this.formatDateParams(params['startDate'], params['endDate']));

    if (_.has(params, 'channel') && _.has(params, 'session')) this.callApi(
            utils.format(constants.endPoints.statsChannelSession, params['channel'], params['session']), params, cb);
    else if (_.has(params, 'channel'))
        this.callApi(utils.format(constants.endPoints.statsChannel, params['channel']), params, cb);
    else this.callApi(constants.endPoints.stats, params, cb);
};


/**
 * MessageBusFeedbackClient provides methods to feedback data.
 * @class MessageBusFeedbackClient
 * @constructor
 * @param apiKey {String} Api key
 * @param [uri = constants.uri] {String} Host uri
 * @param [timeout = constants.timeout]{Number} Connection timeout in milliseconds
 */
var MessageBusFeedbackClient = function(apiKey, uri, timeout) {
    MessageBusFeedbackClient.super_.call(this, apiKey, uri, timeout);
};
util.inherits(MessageBusFeedbackClient, MessageBusBase);

/**
 * Gets stats associated with an account, channel, session on a time range.
 * @param [params = null] - {Object} query params
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 var client = new MessageBusFeedbackClient('apiKey');
 var params = {startDate: '2012-12-10', endDate: '2013-01-01', channel:'channelKey', session:'sessionKey'};
 client.getFeedback(params, function (err, resp) {
    if (err) throw err;
    console.info(resp);
});
 */
MessageBusFeedbackClient.prototype.getFeedback = function(params, cb) {
    params = params || {};
    if (_.isFunction(params)) {
        cb = params;
        params = {};
    }
    params = _.extend(_.omit(params, 'startDate', 'endDate'), this.formatDateParams(
        params['startDate'], params['endDate']));
    if (_.has(params, 'channel') && _.has(params, 'session'))
        this.callApi(utils.format(constants.endPoints.feedbackChannelSession, params['channel'], params['session']), params, cb);
    else if (_.has(params, 'channel'))
        this.callApi(utils.format(constants.endPoints.feedbackChannel, params['channel']), params, cb);
    else this.callApi(constants.endPoints.feedback, params, cb);
};

/**
 * MessageBusTemplatesClient sends email using server side templates.
 * @class MessageBusTemplatesClient
 * @constructor
 * @param apiKey {String} Api key
 * @param [uri = constants.templateUri] {String} Host uri
 * @param [timeout = constants.timeout] {Number} Connection timeout in milliseconds
 * @example
 var client = new MessageBusTemplatesClient('apiKey');
 */
var MessageBusTemplatesClient = function(apiKey, uri, timeout) {
    uri = uri || constants.templateUri;
    MessageBusTemplatesClient.super_.call(this, apiKey, uri, timeout);
};

util.inherits(MessageBusTemplatesClient, MessageBusBase);

/**
 * Create a server side email template
 * @param {Object} template
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 *
 var template = {
            "toName": "{{rcpt_name}}",
            "toEmail": "{{rcpt_email}}",
            "fromName": "{{sender_name}}",
            "fromEmail": "{{sender_email}}",
            "returnPath": "{{return_path}}",
            "subject": "Yay! {{{date_val}}}",
            "plaintextBody": " {{rcpt_name}}!",
            "htmlBody": "{{{html_blob}}} {{date_val}}",
            "sessionKey": "{{session_key}}",
            "customHeaders": {
            }
        };
 var client = new MessageBusTemplatesClient('apiKey');
 client.createTemplate(template, function (err, resp) {
    if (err) throw err;
    console.info(resp.templateKey);
});
 */
MessageBusTemplatesClient.prototype.createTemplate = function(template, cb) {
    this.callApi(constants.endPoints.templates, template, 'POST', cb);
};


/**
 * Send emails via the Message Bus API.
 * @param template - {String} Template key
 * @param messages - {Array} Messages to send
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 var client = new MessageBusTemplatesClient('apiKey');
 var messages = [{
        rcpt_name: "Ted",
        rcpt_email: "ted@example.com",
        sender_name: "Bob",
        sender_email: "bob@example.com",
        return_path: "noreply@example.com",
        session_key: 'DEFAULT',
        x_value: 'some_val'}];

 client.sendMessages('templateKey', messages, function (err, resp) {
    if (err) throw err;
    console.info('Emails sent:', resp['successCount']);
    console.info('Emails failed:', resp['failureCount']);
});
 */
MessageBusTemplatesClient.prototype.sendMessages = function(template, messages,
    cb) {
    if (_.isArray(messages) && messages.length <= constants.maxTemplateMessages)
        this.callApi(constants.endPoints.templateEmailSend, {
            templateKey: template,
            messages: messages
        }, 'POST', cb);
    else {
        var err = new Error(util.format(
            'messages must be an array with maximum of %s messages', constants.maxTemplateMessages));
        if (cb) cb(err);
        else throw err;
    }
};

/**
 * Get a saved template
 * @param {String} template - template key
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 var client = new MessageBusTemplatesClient('apiKey');

 client.getTemplate('templateKey', function (err, resp) {
    if (err) throw err;
    console.info(resp['template']);
});
 */
MessageBusTemplatesClient.prototype.getTemplate = function(template, cb) {
    this.callApi(util.format(constants.endPoints.template, template), cb);
};

/**
 * Get a saved template
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 * @example
 var client = new MessageBusTemplatesClient('apiKey');

 client.getTemplates('templateKey', function (err, resp) {
    if (err) throw err;
    console.info(resp['template']);
});
 */
MessageBusTemplatesClient.prototype.getTemplates = function(cb) {
    this.callApi(constants.endPoints.templates, cb);
};

/**
 * Get the version of the Templates API.
 * @param [cb = null] - {Function} Function callback(err, resp) {}
 */
MessageBusTemplatesClient.prototype.getVersion = function(cb) {
    this.callApi(constants.endPoints.templatesVersion, cb);
};

exports.MessageBusAPIClient = MessageBusAPIClient;
exports.MessageBusStatsClient = MessageBusStatsClient;
exports.MessageBusFeedbackClient = MessageBusFeedbackClient;
exports.MessageBusTemplatesClient = MessageBusTemplatesClient;
