/**
 * @copyright
 * Copyright 2013 Mail Bypass, Inc.
 *
 * @license
 * Licensed under the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain
 * a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */
var mb = require('../lib/messagebus'),
    request = require('request'),
    _ = require('underscore'),
    assert = require('assert'),
    nock = require('nock'),
    util = require('util'),
    Faker = require('Faker');

var apiKey = 'apiKey';
var uri = 'https://api-v4.messagebus.com';

var message = {
    toName: Faker.Name.findName(),
    toEmail: Faker.Internet.email(),
    fromName: Faker.Name.findName(),
    fromEmail: Faker.Internet.email(),
    subject: Faker.Lorem.sentence(),
    plaintextBody: Faker.Lorem.paragraphs(),
    htmlBody: util.format('<p>%s</p>', Faker.Lorem.paragraphs())
};

describe('Message Bus API - v4', function() {
    describe('api - send message', function() {
        it('202 - single message', function(done) {
            var res = {
                failureCount: 0,
                statusCode: 202,
                results: [{
                        messageStatus: 0,
                        toEmail: "bob@example.com",
                        messageId: "fbeeb6e0838911e289f5bc764e049d62"
                    }
                ],
                successCount: 1,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: "."
            };

            var client = new mb.MessageBusAPIClient(apiKey);
            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .matchHeader('Content-Type', 'application/json; charset=utf-8')
                .post('/api/v4/message/email/send', {
                messages: [message]
            })
                .reply(202, res);
            client.sendMessages([message], function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 202);
                done();
            });
        });

        it('202 - multiple messages', function(done) {
            var res = {
                failureCount: 0,
                statusCode: 202,
                results: [{
                        messageStatus: 0,
                        toEmail: "bob@example.com",
                        messageId: "fbeeb6e0838911e289f5bc764e049d62"
                    }, {
                        messageStatus: 0,
                        toEmail: "bob@example.com",
                        messageId: "fbeeb6e0838911e289f5bc764e049d62"
                    }
                ],
                successCount: 2,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: "."
            };

            var client = new mb.MessageBusAPIClient(apiKey);
            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .matchHeader('Content-Type', 'application/json; charset=utf-8')
                .post('/api/v4/message/email/send', {
                messages: [message, message]
            })
                .reply(202, res);
            client.sendMessages([message, message], function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 202);
                assert.ok(_.isArray(resp.results));
                done();
            });
        });

        it('400 (non 2xx) - multiple messages', function(done) {
            var res = {
                statusCode: 400,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: "."
            };

            var client = new mb.MessageBusAPIClient(apiKey);
            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .matchHeader('Content-Type', 'application/json; charset=utf-8')
                .post('/api/v4/message/email/send', {
                messages: [message, message]
            })
                .reply(400, res);
            client.sendMessages([message, message], function(err, resp) {
                assert.ok(_.isUndefined(resp));
                assert.ok(err);
                assert.ok(err.statusCode, 400);
                done();
            });
        });

        it('207 - multiple messages', function(done) {
            var res = {
                failureCount: 1,
                statusCode: 202,
                results: [{
                        messageStatus: 0,
                        toEmail: "bob@example.com",
                        messageId: "fbeeb6e0838911e289f5bc764e049d61"
                    }, {
                        messageStatus: 1000,
                        toEmail: "bob@example.com",
                        messageId: "fbeeb6e0838911e289f5bc764e049d62"
                    }
                ],
                successCount: 1,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: "."
            };

            var client = new mb.MessageBusAPIClient(apiKey);
            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .matchHeader('Content-Type', 'application/json; charset=utf-8')
                .post('/api/v4/message/email/send', {
                messages: [message, message]
            })
                .reply(207, res);
            client.sendMessages([message, message], function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 207);
                assert.ok(resp.failureCount, 1);
                assert.ok(resp.successCount, 1);
                assert.ok(_.isArray(resp.results));
                done();
            });
        });

        it('network error - exception thrown', function(done) {
            var client = new mb.MessageBusAPIClient(apiKey, 'https://qwerty.qwerty.com');
            client.sendMessages([message, message], function(err, resp) {
                assert.ok(_.isUndefined(resp));
                assert.ok(err);
                done();
            });
        });
    });
    describe('api - other', function() {
        it('200 - get channels', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.581Z"
            };

            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .get('/api/v4/channels?')
                .reply(200, res);

            var client = new mb.MessageBusAPIClient(apiKey);
            client.getChannels(function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                done();
            });
        });
        it('200 - get sessions', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.581Z"
            };

            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .get('/api/v4/channel/someChannel/sessions?')
                .reply(200, res);

            var client = new mb.MessageBusAPIClient(apiKey);
            client.getChannelSessions('someChannel', function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                done();
            });
        });
        it('200 - get channel config', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.581Z"
            };

            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .get('/api/v4/channel/someChannel/config?')
                .reply(200, res);

            var client = new mb.MessageBusAPIClient(apiKey);
            client.getChannelConfig('someChannel', function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                done();
            });
        });
        it('201 - rename session', function(done) {
            var res = {
                statusCode: 201,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.581Z"
            };

            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .put('/api/v4/channel/someChannel/session/someSession/rename', {
                sessionName: 'newName'
            })
                .reply(201, res);

            var client = new mb.MessageBusAPIClient(apiKey);
            client.sessionRename('someChannel', 'someSession', 'newName', function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 201);
                done();
            });
        });
        it('201 - create session', function(done) {
            var res = {
                statusCode: 202,
                statusMessage: "",
                statusTime: "2011-09-19T22:40:45.123Z",
                sessionName: "new session name",
                sessionKey: ""
            };

            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .post('/api/v4/channel/someChannel/sessions', {
                sessionName: 'newName'
            })
                .reply(202, res);

            var client = new mb.MessageBusAPIClient(apiKey);
            client.createSession('someChannel', 'newName', function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 201);
                done();
            });
        });
        it('200 - get version', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: "Version",
                version: "1.1.13.6",
                statusTime: "2013-03-19T00:12:15.955Z"
            }

            var client = new mb.MessageBusAPIClient(apiKey);
            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .get('/api/version?')
                .reply(200, res);
            client.getVersion(function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                assert.ok(resp.version, "1.13.6");
                done();
            });
        });

    });
    describe('date handling', function() {
        it('format date', function(done) {
            var client = new mb.MessageBusAPIClient(apiKey);
            var dates = client.formatDateParams('1970-01-01T02:47:13.332Z', '1971-01-01T02:47:13.332Z');
            assert.ok(dates.startDate, '1970-01-01T02:47:13.332Z');
            assert.ok(dates.endDate, '1971-01-01T02:47:13.332Z');
            done();
        });

        it('format date - invalid end date', function(done) {
            var client = new mb.MessageBusAPIClient(apiKey);
            var dates = client.formatDateParams('1970-01-01T02:47:13.332Z', 'foo');
            assert.ok(dates.startDate, '1970-01-01T02:47:13.332Z');
            assert.ok(!dates.hasOwnProperty('endDate'));
            done();
        });
        it('format date - numeric date', function(done) {
            var client = new mb.MessageBusAPIClient(apiKey);
            var dates = client.formatDateParams(0, 'foo');
            assert.ok(dates.startDate, '2000-01-01T08:00:00.000Z');
            assert.ok(!dates.hasOwnProperty('endDate'));
            done();
        });

        it('format date - date objs', function(done) {
            var client = new mb.MessageBusAPIClient(apiKey);
            var now = new Date();
            var dates = client.formatDateParams(now, now);
            assert.ok(dates.startDate, now.toISOString());
            assert.ok(dates.endDate, now.toISOString());
            done();
        });

        it('format date - null, null', function(done) {
            var client = new mb.MessageBusAPIClient(apiKey);
            var dates = client.formatDateParams(null, null);
            assert.ok(dates, {});
            assert.ok(!dates.hasOwnProperty('startDate'));
            assert.ok(!dates.hasOwnProperty('endDate'));

            var dates2 = client.formatDateParams();
            assert.ok(dates2, {});
            assert.ok(!dates2.hasOwnProperty('startDate'));
            assert.ok(!dates2.hasOwnProperty('endDate'));

            var dates2 = client.formatDateParams('', '');
            assert.ok(dates2, {});
            assert.ok(!dates2.hasOwnProperty('startDate'));
            assert.ok(!dates2.hasOwnProperty('endDate'));

            done();


        });

    });
    describe('stats', function() {
        it('200 - get statistics', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.581Z"
            };

            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .get('/api/v4/stats/email?')
                .reply(200, res);

            var client = new mb.MessageBusStatsClient(apiKey);
            client.getStats(function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                done();
            });
        });
        it('200 - get statistics with null dates', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.581Z"
            };

            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .get('/api/v4/stats/email?')
                .reply(200, res);

            var client = new mb.MessageBusStatsClient(apiKey);
            client.getStats({
                startDate: '',
                endDate: ''
            }, function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                done();
            });
        });
        it('200 - get statistics with both numeric dates', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.591Z"
            };

            nock('https://api-v4.messagebus.com')
            //.get('/api/v4/stats/email?startDate=1970-01-01T00%3A00%3A00.042Z&endDate=1970-01-01T00%3A00%3A04.200Z')
            .get('/api/v4/stats/email?startDate=1970-01-01T00%3A00%3A00Z&endDate=1970-01-01T00%3A00%3A04Z')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .reply(200, JSON.stringify(res));

            var client = new mb.MessageBusStatsClient(apiKey);
            client.getStats({
                startDate: 42,
                endDate: 4200
            }, function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                assert.ok(resp.statusTime, "2013-03-02T22:39:03.591Z");
                done();
            });
        });

        it('200 - get statistics with single (startDate) numeric dates', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.591Z"
            };

            nock('https://api-v4.messagebus.com')
                .get('/api/v4/stats/email?startDate=1970-01-01T00%3A00%3A00Z')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .reply(200, JSON.stringify(res));

            var client = new mb.MessageBusStatsClient(apiKey);
            client.getStats({
                startDate: 42
            }, function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                assert.ok(resp.statusTime, "2013-03-02T22:39:03.591Z");
                done();
            });
        });

        it('200 - get statistics with single (endDate) numeric dates', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.591Z"
            };

            nock('https://api-v4.messagebus.com')
                .get('/api/v4/stats/email?endDate=1970-01-01T00%3A00%3A00Z')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .reply(200, JSON.stringify(res));

            var client = new mb.MessageBusStatsClient(apiKey);
            client.getStats({
                endDate: 42
            }, function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                assert.ok(resp.statusTime, "2013-03-02T22:39:03.591Z");
                done();
            });
        });

        it('200 - get statistics with date obj', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.591Z"
            };

            var now = new Date('2013-04-01T22:23:20Z');
            var client = new mb.MessageBusStatsClient(apiKey);

            nock('https://api-v4.messagebus.com')
                .get('/api/v4/stats/email?endDate=2013-04-01T22%3A23%3A20Z')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .reply(200, JSON.stringify(res));

            client.getStats({
                endDate: now
            }, function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                assert.ok(resp.statusTime, "2013-03-02T22:39:03.591Z");
                done();
            });
        });

        it('400 - resp', function(done) {
            var res = {
                statusCode: 400,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.591Z"
            };
            var now = new Date('2013-04-01T22:29:21Z');
            nock('https://api-v4.messagebus.com')
                .get('/api/v4/stats/email?endDate=2013-04-01T22%3A29%3A21Z')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .reply(200, JSON.stringify(res));

            var client = new mb.MessageBusStatsClient(apiKey);
            client.getStats({
                endDate: now
            }, function(err, resp) {
                assert.ok(!_.isNull(err));
                assert.ok(err);
                assert.ok(err.statusCode, 400);
                assert.ok(err.statusTime, "2013-03-02T22:39:03.591Z");
                done();
            });
        });
    });
    describe('feedback', function() {
        it('200 - get feedback', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.581Z"
            };

            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .get('/api/v4/feedback?')
                .reply(200, res);

            var client = new mb.MessageBusFeedbackClient(apiKey);
            client.getFeedback(function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                done();
            });
        });
        it('200 - get feedback with null dates', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.581Z"
            };

            nock('https://api-v4.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .get('/api/v4/feedback?')
                .reply(200, res);

            var client = new mb.MessageBusFeedbackClient(apiKey);
            client.getFeedback({
                startDate: '',
                endDate: ''
            }, function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                done();
            });
        });
        it('200 - get feedback with both numeric dates and useSendTime', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.591Z"
            };

            nock('https://api-v4.messagebus.com')
                .get('/api/v4/feedback?useSendTime=true&startDate=1970-01-01T00%3A00%3A00Z&endDate=1970-01-01T00%3A00%3A04Z')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .reply(200, res);

            var client = new mb.MessageBusFeedbackClient(apiKey);
            client.getFeedback({
                useSendTime: true,
                startDate: 42,
                endDate: 4200
            }, function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                assert.ok(resp.statusTime, "2013-03-02T22:39:03.591Z");
                done();
            });
        });

        it('200 - get feedback with single (startDate) numeric dates and scope = all', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.591Z"
            };

            nock('https://api-v4.messagebus.com')
                .get('/api/v4/feedback?scope=all&startDate=1970-01-01T00%3A00%3A00Z')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .reply(200, res);

            var client = new mb.MessageBusFeedbackClient(apiKey);
            client.getFeedback({
                scope: 'all',
                startDate: 42
            }, function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                assert.ok(resp.statusTime, "2013-03-02T22:39:03.591Z");
                done();
            });
        });

        it('200 - get feedback with single (endDate) numeric dates', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.591Z"
            };

            nock('https://api-v4.messagebus.com, scope and useSendTime')
                .get('/api/v4/feedback?scope=unsubs&useSendTime=true&endDate=1970-01-01T00%3A00%3A00Z')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .reply(200, JSON.stringify(res));

            var client = new mb.MessageBusFeedbackClient(apiKey);
            client.getFeedback({
                scope: 'unsubs',
                useSendTime: true,
                endDate: 42
            }, function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                assert.ok(resp.statusTime, "2013-03-02T22:39:03.591Z");
                done();
            });
        });

        it('200 - get feedback with date obj', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.591Z"
            };

            var now = new Date('2013-04-01T22:23:20Z');

            nock('https://api-v4.messagebus.com')
                .get('/api/v4/feedback?endDate=2013-04-01T22%3A23%3A20Z')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .reply(200, JSON.stringify(res));

            var client = new mb.MessageBusFeedbackClient(apiKey);
            client.getFeedback({
                endDate: now
            }, function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                assert.ok(resp.statusTime, "2013-03-02T22:39:03.591Z");
                done();
            });
        });

        it('400 - resp', function(done) {
            var res = {
                statusCode: 400,
                statusMessage: '',
                statusTime: "2013-03-02T22:39:03.591Z"
            };
            var now = new Date('2013-04-01T22:29:21Z');
            nock('https://api-v4.messagebus.com')
                .get('/api/v4/feedback?endDate=2013-04-01T22%3A29%3A21Z')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .reply(200, JSON.stringify(res));

            var client = new mb.MessageBusFeedbackClient(apiKey);
            client.getFeedback({
                endDate: now
            }, function(err, resp) {
                assert.ok(!_.isNull(err));
                assert.ok(err);
                assert.ok(err.statusCode, 400);
                assert.ok(err.statusTime, "2013-03-02T22:39:03.591Z");
                done();
            });
        });
    });
    describe('template - send message', function() {
        it('202 - single template message', function(done) {
            var res = {
                failureCount: 0,
                statusCode: 202,
                results: [{
                        messageStatus: 0,
                        toEmail: "bob@example.com",
                        messageId: "fbeeb6e0838911e289f5bc764e049d62"
                    }
                ],
                successCount: 1,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: "."
            };

            var client = new mb.MessageBusTemplatesClient(apiKey);
            nock('https://templates-v4-jy01-prod.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .matchHeader('Content-Type', 'application/json; charset=utf-8')
                .post('/api/v4/templates/email/send', {
                templateKey: 'templateKey',
                messages: [message]
            })
                .reply(202, res);
            client.sendMessages('templateKey', [message], function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 202);
                done();
            });
        });

        it('202 - multiple messages', function(done) {
            var res = {
                failureCount: 0,
                statusCode: 202,
                results: [{
                        messageStatus: 0,
                        toEmail: "bob@example.com",
                        messageId: "fbeeb6e0838911e289f5bc764e049d62"
                    }, {
                        messageStatus: 0,
                        toEmail: "bob@example.com",
                        messageId: "fbeeb6e0838911e289f5bc764e049d62"
                    }
                ],
                successCount: 2,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: "."
            };

            var client = new mb.MessageBusTemplatesClient(apiKey);
            nock('https://templates-v4-jy01-prod.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .matchHeader('Content-Type', 'application/json; charset=utf-8')
                .post('/api/v4/templates/email/send', {
                templateKey: 'templateKey',
                messages: [message, message]
            })
                .reply(202, res);
            client.sendMessages('templateKey', [message, message], function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 202);
                assert.ok(_.isArray(resp.results));
                done();
            });
        });

        it('400 (non 2xx) - multiple messages', function(done) {
            var res = {
                statusCode: 400,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: "."
            };

            var client = new mb.MessageBusTemplatesClient(apiKey);
            nock('https://templates-v4-jy01-prod.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .matchHeader('Content-Type', 'application/json; charset=utf-8')
                .post('/api/v4/templates/email/send', {
                templateKey: 'templateKey',
                messages: [message, message]
            })
                .reply(400, res);
            client.sendMessages('templateKey', [message, message], function(err, resp) {
                assert.ok(_.isUndefined(resp));
                assert.ok(err);
                assert.ok(err.statusCode, 400);
                done();
            });
        });

        it('207 - multiple messages', function(done) {
            var res = {
                failureCount: 1,
                statusCode: 202,
                results: [{
                        messageStatus: 0,
                        toEmail: "bob@example.com",
                        messageId: "fbeeb6e0838911e289f5bc764e049d61"
                    }, {
                        messageStatus: 1000,
                        toEmail: "bob@example.com",
                        messageId: "fbeeb6e0838911e289f5bc764e049d62"
                    }
                ],
                successCount: 1,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: "."
            };

            var client = new mb.MessageBusTemplatesClient(apiKey);
            nock('https://templates-v4-jy01-prod.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .matchHeader('Content-Type', 'application/json; charset=utf-8')
                .post('/api/v4/templates/email/send', {
                templateKey: 'templateKey',
                messages: [message, message]
            })
                .reply(207, res);
            client.sendMessages('templateKey', [message, message], function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 207);
                assert.ok(resp.failureCount, 1);
                assert.ok(resp.successCount, 1);
                assert.ok(_.isArray(resp.results));
                done();
            });
        });

        it('network error - exception thrown', function(done) {
            var client = new mb.MessageBusTemplatesClient(apiKey, 'https://qwerty.qwerty.com');
            client.sendMessages('templateKey', [message, message], function(err, resp) {
                assert.ok(_.isUndefined(resp));
                assert.ok(err);
                done();
            });
        });
    });
    describe('template - create/get/list template(s)', function() {
        it('201 - template create', function(done) {
            var res = {
                templateKey: "550e8400-e29b-41d4-a716-446655440000",
                statusCode: 201,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: "."
            };

            var client = new mb.MessageBusTemplatesClient(apiKey);
            nock('https://templates-v4-jy01-prod.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .matchHeader('Content-Type', 'application/json; charset=utf-8')
                .post('/api/v4/templates', {
                foo: 42
            })
                .reply(201, res);
            client.createTemplate({
                foo: 42
            }, function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 201);
                assert.ok(resp.templateKey, "550e8400-e29b-41d4-a716-446655440000");
                done();
            });
        });
        it('400 - template create', function(done) {
            var res = {
                statusCode: 400,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: "."
            };

            var client = new mb.MessageBusTemplatesClient(apiKey);
            nock('https://templates-v4-jy01-prod.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .matchHeader('Content-Type', 'application/json; charset=utf-8')
                .post('/api/v4/templates', {
                foo: 42
            })
                .reply(400, res);
            client.createTemplate({
                foo: 42
            }, function(err, resp) {
                assert.ok(!_.isNull(err));
                assert.ok(err.statusCode, 400);
                done();
            });
        });
        it('200 - template get', function(done) {
            var res = {
                statusCode: 200,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: ".",
                template: {}
            };

            var client = new mb.MessageBusTemplatesClient(apiKey);
            nock('https://templates-v4-jy01-prod.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .get('/api/v4/template/550e8400-e29b-41d4-a716-446655440000?')
                .reply(200, res);
            client.getTemplate("550e8400-e29b-41d4-a716-446655440000", function(err, resp) {
                assert.ok(!_.isNull(resp));
                assert.ok(resp.statusCode, 200);
                done();
            });
        });
        it('200 - templates get', function(done) {
            var res = {
                statusCode: 200,
                statusTime: "2013-03-02T22:39:03.581Z",
                statusMessage: ".",
                templates: []
            };

            var client = new mb.MessageBusTemplatesClient(apiKey);
            nock('https://templates-v4-jy01-prod.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .get('/api/v4/templates?')
                .reply(200, res);
            client.getTemplates(function(err, resp) {
                assert.ok(!_.isNull(resp));
                assert.ok(resp.statusCode, 200);
                done();
            });
        });

    });
    describe('template - version', function() {
        it('200 - get version', function(done) {
            var res = {
                statusCode: 200,
                statusMessage: "Version",
                version: "1.13.6",
                statusTime: "2013-03-19T00:12:15.955Z"
            }

            var client = new mb.MessageBusTemplatesClient(apiKey);
            nock('https://templates-v4-jy01-prod.messagebus.com')
                .matchHeader('User-Agent', /MessageBusAPI:.*/)
                .matchHeader('X-MessageBus-Key', apiKey)
                .get('/api/v4/templates/version?')
                .reply(200, res);
            client.getVersion(function(err, resp) {
                assert.ok(_.isNull(err));
                assert.ok(resp);
                assert.ok(resp.statusCode, 200);
                assert.ok(resp.version, "1.13.6");
                done();
            });
        });
    });
});
